"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const emitter_1 = require("./emitter");
const validator_1 = require("./validator");
const errors_1 = require("../errors");
class Context extends emitter_1.Emitter {
    constructor(id, events) {
        super(events);
        this.plugins = new Map();
        this.components = new Map();
        if (!validator_1.Validator.isValidId(id)) {
            throw new Error(errors_1.EngineError.InvalidEngineIDFormat);
        }
        this.id = id;
    }
    use(plugin, options) {
        if (this.plugins.has(plugin.name)) {
            throw new Error(errors_1.EngineError.PluginAlreadyExists);
        }
        plugin.install(this, options || {});
        this.plugins.set(plugin.name, options);
    }
    register(component) {
        if (this.components.has(component.name)) {
            throw new Error(errors_1.EngineError.ComponentAlreadyExists);
        }
        this.components.set(component.name, component);
        this.trigger('componentregister');
    }
    destroy() {
        this.trigger('destroy');
    }
}
exports.Context = Context;
//# sourceMappingURL=context.js.map